/*
 * @(#) header.h 1.4, last edit: 2/10/94 12:11:42
 * @(#) (C) 1993/94 Tobias Bading (bading@cs.tu-berlin.de)
 * @(#) Berlin University of Technology
 */

#ifndef HEADER_H
#define HEADER_H

#include "all.h"
#include "ibitstream.h"
#include "crc.h"


enum e_mode { stereo, joint_stereo, dual_channel, single_channel  };
enum e_sample_frequency { fourtyfour_point_one, fourtyeight, thirtytwo };


// Class for extraction information from a frame header:
class Header
{
  static const unsigned frequencies[3];
  unsigned h_layer, h_protection_bit, h_bitrate_index, h_padding_bit, h_mode_extension;
  e_mode   h_mode;
  e_sample_frequency h_sample_frequency;
  unsigned h_number_of_subbands, h_intensity_stereo_bound;
  bool	   h_copyright, h_original;
  Crc16    *crc;
  unsigned short checksum;

  unsigned calculate_framesize ();

public:
			Header (void) { crc = (Crc16 *)0; }
		       ~Header (void) { if (crc) delete crc; }
  bool			read_header (Ibitstream *, Crc16 **);
			// read a 32-bit header from the bitstream

  // functions to query header contents:
  unsigned		layer (void) { return h_layer; };
  unsigned		bitrate_index (void) { return h_bitrate_index; };
  e_sample_frequency	sample_frequency (void) { return h_sample_frequency; };
  unsigned		frequency (void) { return frequencies[h_sample_frequency]; }
  static unsigned	frequency (e_sample_frequency rate) { return frequencies[rate]; }
  e_mode		mode (void) { return h_mode; };
  bool			checksums (void) { return !h_protection_bit; }
  bool			copyright (void) { return h_copyright; }
  bool			original (void) { return h_original; }

  bool			checksum_ok (void) { return checksum == crc->checksum (); }
			// compares computed checksum with stream checksum

  // functions which return header informations as strings:
  const char *		layer_string (void);
  const char *		bitrate_string (void);
  const char *		sample_frequency_string (void);
  const char *		mode_string (void);

  unsigned		number_of_subbands (void) { return h_number_of_subbands; };
			// returns the number of subbands in the current frame
  unsigned		intensity_stereo_bound (void) {return h_intensity_stereo_bound; };
			// (Layer II joint stereo only)
			// returns the number of subbands which are in stereo mode,
			// subbands above that limit are in intensity stereo mode
};

#endif
