/*
 * @(#) ibitstream.c 1.3, last edit: 2/10/94 15:53:14
 * @(#) (C) 1993/94 Tobias Bading (bading@cs.tu-berlin.de)
 * @(#) Berlin University of Technology
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <fcntl.h>
#include <iostream.h>
#include "all.h"
#include "ibitstream.h"


Ibitstream::Ibitstream (int filedescriptor)		// constructor
{
  fd = filedescriptor;
  wordpointer = buffer;
  bitindex = 0;
}


Ibitstream::Ibitstream (const char *filename)		// constructor
{
  if ((fd = open (filename, O_RDONLY)) < 0)
  {
    cerr << "can't open file \"" << filename << "\" for reading!\n";
    exit (1);
  }
  wordpointer = buffer;
  bitindex = 0;
}


Ibitstream::~Ibitstream (void)				// destructor
{
  close (fd);
}


bool Ibitstream::get_header (unsigned *headerstring)
{
  int readvalue;

  if ((readvalue = read (fd, (char *)headerstring, 4)) != 4)
  {
    if (readvalue < 0)
    {
      perror ("read");
      exit (1);
    }
    else if (!readvalue)
      return False;		// EOF
    // header has not been read completely (this may happen while using a pipe)
    int length = 4 - readvalue;
    char *buffer_pos = (char *)buffer;
    do
    {
      buffer_pos += readvalue;
      readvalue = read (fd, buffer_pos, length);
      if (readvalue < 0)
      {
	perror ("read");
	exit (1);
      }
      else if (!readvalue)
	return False;
    }
    while (length -= readvalue);
  }
  return True;
}


bool Ibitstream::read_frame (unsigned bytesize)
{
  int readvalue;

  if (bytesize > (bufferintsize << 2))
  {
    cerr << "Internal error: framelength > bufferlength?!\n";
    exit (1);
  }

  if ((readvalue = read (fd, (char *)buffer, bytesize)) != bytesize)
  {
    if (readvalue < 0)
    {
      perror ("read");
      exit (1);
    }
    else if (!readvalue)
      return False;		// EOF
    // frame has not been read completely (this may happen while using a pipe)
    int length = bytesize - readvalue;
    char *buffer_pos = (char *)buffer;
    do
    {
      buffer_pos += readvalue;
      readvalue = read (fd, buffer_pos, length);
      if (readvalue < 0)
      {
	perror ("read");
	exit (1);
      }
      else if (!readvalue)
	return False;
    }
    while (length -= readvalue);
  }

  wordpointer = buffer;
  bitindex = 0;
  framesize = bytesize;
  return True;
}


unsigned Ibitstream::get_bits (unsigned number_of_bits)
{
  static unsigned bitmask[17] =
  {
    0,	// dummy
    0x00000001, 0x00000003, 0x00000007, 0x0000000F,
    0x0000001F, 0x0000003F, 0x0000007F, 0x000000FF,
    0x000001FF, 0x000003FF, 0x000007FF, 0x00000FFF,
    0x00001FFF, 0x00003FFF, 0x00007FFF, 0x0000FFFF
  };
  unsigned returnvalue;
  unsigned sum = bitindex + number_of_bits;

#ifdef DEBUG
  if (number_of_bits < 1 || number_of_bits > 16)
  {
    cerr << "illegal parameter in Ibitstream::get_bits() !\n";
    exit (1);
  }
#endif

  if (sum <= 32)
  {
    // all bits contained in *wordpointer
    returnvalue = (*wordpointer >> (32 - sum)) & bitmask[number_of_bits];
    if ((bitindex += number_of_bits) == 32)
    {
      bitindex = 0;
      if ((char *)++wordpointer > (char *)buffer + framesize)
      {
	cerr << "Ibitstream::get_bits(): no more bits in buffer!\n";
	exit (1);
      }
    }
    return returnvalue;
  }

  // bits are in *wordpointer and *(wordpointer+1)
  // bitindex has to be > 16
  *((short *)&returnvalue) = *((short *)wordpointer + 1);
  if ((char *)++wordpointer > (char *)buffer + framesize)
  {
    cerr << "Ibitstream::get_bits(): no more bits in buffer!\n";
    exit (1);
  }
  *((short *)&returnvalue + 1) = *((short *)wordpointer);
  returnvalue >>= 48 - sum;	// returnvalue >>= 16 - (number_of_bits - (32 - bitindex))
  returnvalue &= bitmask[number_of_bits];
  bitindex = sum - 32;
  return returnvalue;
}
